require 'go_run/runner'
require 'climate_control'

describe Runner do
  it "runs all commands and records their exit code" do
    config = {
      "some_commands" => { "commands" => ["echo 'command 1'", "echo 'command 2'"] }
    }
    @runner = Runner.new(config)
    @runner.run(["some_commands"])
    expect(@runner.commands.length).to be(2)
    expect(@runner.commands[0]["exit_code"]).to be(0)
    expect(@runner.commands[1]["exit_code"]).to be(0)
  end

  it "stops running commands when a command fails" do
    config = {
      "some_commands" => { "commands" => ["echo 'command 1'", "echo 'command 2'; exit 1", "echo 'command 3'"] }
    }
    @runner = Runner.new(config)
    output = @runner.run(["some_commands"])
    expect(@runner.commands.length).to be(2)
    expect(@runner.commands[0]["exit_code"]).to be(0)
    expect(@runner.commands[1]["exit_code"]).to be(1)
    expect(output).to include("Failed to run command 'echo 'command 2'; exit 1'")
  end

  it "does not run commands if required environment variables are not set" do
    config = {
      "deploy" => {
        "requires" => [{"key" => "ENVIRONMENT"}],
        "commands" => ['echo "deploying $ENVIRONMENT"']
      }
    }
    @runner = Runner.new(config)
    ClimateControl.modify ENVIRONMENT: nil do
      output = @runner.run(["deploy"])
      expect(@runner.commands.length).to be(0)
      expect(output).to include("Environment variable ENVIRONMENT not set")
    end
  end

  it "can set environment variables to a default value" do
    config = {
      "deploy" => {
        "requires" => [{"key" => "ENVIRONMENT", "default" => "some value"}],
        "commands" => ['echo "deploying $ENVIRONMENT"']
      }
    }
    @runner = Runner.new(config)
    ClimateControl.modify ENVIRONMENT: nil do
      output = @runner.run(["deploy"])
      expect(@runner.commands.length).to be(1)
      expect(ENV['ENVIRONMENT']).to eq("some value")
    end
  end

  it "can set environment variables" do
    config = {
      "deploy" => {
        "set" => [{"key" => "ENVIRONMENT", "value" => "dev"}],
        "commands" => ['echo "deploying $ENVIRONMENT"']
      }
    }
    @runner = Runner.new(config)
    ClimateControl.modify ENVIRONMENT: nil do
      output = @runner.run(["deploy"])
      expect(@runner.commands.length).to be(1)
      expect(ENV['ENVIRONMENT']).to eq("dev")
    end
  end

  it "can call other targets" do
    config = {
      "dev" => {
        "set" => [{"key" => "ENVIRONMENT", "value" => "dev"}]
      },
      "deploy" => {
        "call" => [{"target" => "dev"}],
        "commands" => ['echo "deploying $ENVIRONMENT"']
      }
    }
    @runner = Runner.new(config)
    ClimateControl.modify [ENVIRONMENT: nil] do
      output = @runner.run(["deploy"])
      expect(ENV['ENVIRONMENT']).to eq("dev")
      expect(@runner.commands.length).to be(1)
      expect(@runner.commands[0]["exit_code"]).to be(0)
    end
  end

  it "can run groups of targets" do
    config = {
      "dev" => {
        "set" => [{"key" => "ENVIRONMENT", "value" => "dev"}]
      },
      "deploy" => {
        "call" => [{"target" => "dev"}],
        "commands" => ['echo "deploying $ENVIRONMENT"']
      },
      "dev-deploy" => {
        "group" => ["dev", "deploy"]
      }
    }
    @runner = Runner.new(config)
    ClimateControl.modify [ENVIRONMENT: nil] do
      output = @runner.run(["dev-deploy"])
      expect(ENV['ENVIRONMENT']).to eq("dev")
      expect(@runner.commands.length).to be(1)
      expect(@runner.commands[0]["exit_code"]).to be(0)
    end
  end
end
